#The goal here is to produce a plot that has the following features for comparision:
#1/f^6 plot
#Old UIM model
#New UIM Model
#Data that went into model

import numpy as np
import control as ctrl
import matplotlib.pyplot as plt
# import slycot# control.minreal requires slycot. requires linux packages "gfortran liblapack-dev" to build when installing through pip3
import scipy.io as spio
import scipy.signal as signal
import sys
#import fotontst as foton

#resultsDir = SVNROOT+'Runs/'+pars[0].run+'/'+pars[0].IFO+'/Results/FullIFOActuationTFs/'
resultsDir = '/ligo/svncommon/CalSVN/aligocalibration/trunk/Runs/O3/H1/Results/FullIFOActuationTFs/' #For use in my home dir

##########
#Constants
##########


##########
#Functions
##########

#freq resp function for ZPKs stolen from pyDARM scripts - for getting DC gain (/scr/filtermant.py). Vlad made a small bugfix (needed for python3 only?) in the first functional line
#scipy is lazy and does this by converting to zpk to polynomial and it is simply not viable for long lists of Zeroes and Poles
def freqrespZPK(sys, w):
	"""
	Compute the frequency response of a continuous ZPK filter
	"""
	s = 1j * np.complex128(w)
	numPoles = len(sys.poles)
	g = sys.gain * np.ones(len(w),dtype='complex128')

	sortedPolesIdx = np.argsort(abs(sys.poles))
	sortedZerosIdx = np.argsort(abs(sys.zeros))

	for i in range(0, numPoles):
		if i < len(sys.zeros):
			tmp = s - sys.zeros[sortedZerosIdx[i]]
		else:
			tmp = 1
		g = g * tmp / (s - sys.poles[sortedPolesIdx[i]])

	return g

def find_nearest(array, value):
	array = np.asarray(array)
	idx = (np.abs(array - value)).argmin()
	return idx

def tfdiff(reference_complex_tf,new_complex_tf):
	gain_reference_complex_tf = [None] *len(fdata)
	pha_reference_complex_tf = [None] *len(fdata)
	ratios = [None] *len(fdata)
	magdiff = [None] *len(fdata)
	phadiff = [None] *len(fdata)
	for row in range(len(fdata)):
		gain_reference_complex_tf[row] = np.abs(reference_complex_tf[row])
		pha_reference_complex_tf[row] = np.angle(reference_complex_tf[row], deg=True)
		ratios[row] = new_complex_tf[row] / reference_complex_tf[row]
		magdiff[row] = np.abs(ratios[row])
		phadiff[row] = np.angle(ratios[row], deg=True) #this wraps but is ok for tiny angles
	#Convert to Deg, and Unwrap phases... or as presently: don't
	#pha_reference_complex_tf = (180.0 / np.pi) * pha_reference_complex_tf#np.unwrap(pha_reference_complex_tf)* 180.0 / np.pi
	return np.array(gain_reference_complex_tf), np.array(pha_reference_complex_tf), np.array(magdiff), np.array(phadiff)

#########################
#Import filters
#########################

#Data
fdata, reallist, imaglist, uncertainty= np.loadtxt(resultsDir+'2019-11-20_H1SUSETMX_L1_Data.txt').T

#1/f^6 plot
z = np.array([])
p = 2*np.pi* np.array([0.01,0.01,0.01,0.01,0.01,0.01])
k = 46.5

#Old UIM model
quadnoUIMdyn = spio.loadmat('/ligo/svncommon/CalSVN/aligocalibration/trunk/Common/pyDARM/matlab_scripts/H1susdata_O3.mat')
#quadnoUIMdyn = spio.loadmat('/ligo/svncommon/CalSVN/aligocalibration/trunk/Common/pyDARM/matlab_scripts/20200107_H1_EX_O3_susdata_nouim.mat')

UIMz0 = quadnoUIMdyn['UIMz'].T[0]
UIMp0 = quadnoUIMdyn['UIMp'].T[0]
UIMk0 = quadnoUIMdyn['UIMk'].T[0][0]

#New UIM Model
quadfull = spio.loadmat('/ligo/svncommon/CalSVN/aligocalibration/trunk/Common/pyDARM/matlab_scripts/20200107_H1_EX_O3_susdata.mat')
UIMz = quadfull['UIMz'].T[0]
UIMp = quadfull['UIMp'].T[0]
UIMk = quadfull['UIMk'].T[0][0]

#Old CALCS filter - has huge amout of data - prune to only keed data nearest to the fdata vector
f, foton_old_real, foton_old_imag = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/1/current_foton.txt').T


#New CALCS filter - has huge amout of data - prune to only keed data nearest to the fdata vector
f, foton_new_real, foton_new_imag = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/1/new_foton.txt').T

##############################################################################
#Evaluate freqResp
##############################################################################

#First find the indicies that are closest to the data, in the foton filter files.
goodInd = [None] *len(fdata)
for row in range(len(fdata)): # Good indicies are good for both files, since they were generated with the same parameters from foton.
	goodInd[row] = find_nearest(f,fdata[row])

f = f[goodInd]
foton_old_real = foton_old_real[goodInd]
foton_old_imag = foton_old_imag[goodInd]
foton_new_real = foton_new_real[goodInd]
foton_new_imag = foton_new_imag[goodInd]
foton_old = [None] *len(fdata)
foton_new = [None] *len(fdata)
ref_data =  [None] *len(fdata)
for row in range(len(fdata)):
	foton_old[row] = complex(foton_old_real[row], foton_old_imag[row])
	foton_new[row] = complex(foton_new_real[row], foton_new_imag[row])
	ref_data[row] = complex(reallist[row], imaglist[row])
pyDARM_old = freqrespZPK(signal.ZerosPolesGain(UIMz0,UIMp0,UIMk0), 2*np.pi*fdata)
pyDARM_new = freqrespZPK(signal.ZerosPolesGain(UIMz,UIMp,UIMk), 2*np.pi*fdata)
ONEonf6 = freqrespZPK(signal.ZerosPolesGain(z,p,k), 2*np.pi*fdata)


##############################################################################
#Calcualte gains and phases
##############################################################################

gain_pyDARM_old, pha_pyDARM_old, magdiff_pyDARM_old, phadiff_pyDARM_old = tfdiff( pyDARM_old, ref_data)
gain_pyDARM_new, pha_pyDARM_new, magdiff_pyDARM_new, phadiff_pyDARM_new = tfdiff( pyDARM_new, ref_data)
gain_foton_old, pha_foton_old, magdiff_foton_old, phadiff_foton_old = tfdiff( foton_old, ref_data)
gain_foton_new, pha_foton_new, magdiff_foton_new, phadiff_foton_new = tfdiff( foton_new, ref_data)
gain_1onf6, pha_1onf6, magdiff_1onf6, phadiff_1onf6 = tfdiff( ONEonf6, ref_data)
gain_data, pha_data, magdiff_silly, phadiff_silly = tfdiff( ref_data, ref_data)

############################
#Plot the response functions
############################

#The order of the below is in order of increasing 'goodness', thus plotting the best points on the very top where they can be seen as such.
plots = ['1/f^6', 'foton_pre_20200113', 'pyDARM_20190909', 'foton_post_20200113', 'pyDARM_20200103']
gains = [gain_1onf6,gain_foton_old, gain_pyDARM_old, gain_foton_new, gain_pyDARM_new]
phases = [pha_1onf6, pha_foton_old, pha_pyDARM_old, pha_foton_new, pha_pyDARM_new]
magdiff = [magdiff_1onf6, magdiff_foton_old, magdiff_pyDARM_old, magdiff_foton_new, magdiff_pyDARM_new]
phadiff = [phadiff_1onf6, phadiff_foton_old, phadiff_pyDARM_old, phadiff_foton_new, phadiff_pyDARM_new]
nDataSets = len(plots)

freq_plt_vec = [[5, 40], [40, 100], [100, 200], [200, 400], [400, 550]]#, [550, 1000]] ... last one is kina useless

for vec in freq_plt_vec:
	index= (vec[0] < fdata) & (fdata <= vec[1])
	cm = plt.get_cmap('gist_rainbow')
	plt.figure(figsize=(15,10)).subplots_adjust(hspace=0.3, wspace=0.2)
	plt.subplot(221)
	plt.xscale('log')
	plt.yscale('log')
	for n in range(len(plots)): 
		plt.scatter(fdata[index],(gains[n])[index], label=plots[n], color=cm(n/nDataSets))
	plt.scatter(fdata[index],np.array(gain_data)[index], label='Data', color='black',marker='x', alpha=.3)
	if vec == [100, 200]: #some custom goodness to make the add some y axis ticks, where there were none before for one specific plot
		plt.xticks([100,120,140,160,180,200])
	plt.ylabel('Magnitude[abs]')
	plt.xlabel('Frequency [Hz]')
	plt.grid(True, 'both')
	plt.title('Magnitude of UIM response')
	plt.legend()

	plt.subplot(223)
	plt.xscale('log')
	for n in range(len(plots)): 
		plt.scatter(fdata[index],(phases[n])[index], label=plots[n], color=cm(n/nDataSets))
	plt.scatter(fdata[index],np.array(pha_data)[index], label='Data', color='black',marker='x', alpha=.3)
	if vec == [100, 200]: #some custom goodness to make the add some y axis ticks, where there were none before for one specific plot
		plt.xticks([100,120,140,160,180,200])
	plt.ylabel('Phase[rad]')
	plt.xlabel('Frequency [Hz]')
	plt.grid(True, 'both')
	plt.title('Phase of UIM response')
	#plt.legend()

	plt.subplot(222)
	plt.xscale('log')
	plt.yscale('log')
	for n in range(len(plots)): 
		plt.scatter(fdata[index],(magdiff[n])[index], label=plots[n], color=cm(n/nDataSets))
	plt.grid(True, 'both')
	if vec==[5, 40]: #some custom goodness to make the y axist ticks better for the first frequency block
		plt.yticks([0.8,0.9,1,1.1,1.2])
	if vec == [100, 200]: #some custom goodness to make the add some y axis ticks, where there were none before for one specific plot
		plt.xticks([100,120,140,160,180,200])
	plt.ylabel('Magnitude Ratio')
	plt.xlabel('Frequency [Hz]')
	plt.title('Magnitude Ratio from reference')

	plt.subplot(224)
	plt.xscale('log')
	for n in range(len(plots)): 
		plt.scatter(fdata[index],(phadiff[n])[index], label=plots[n], color=cm(n/nDataSets))
	if vec == [100, 200]: #some custom goodness to make the add some y axis ticks, where there were none before for one specific plot
		plt.xticks([100,120,140,160,180,200])
	plt.grid(True, 'both')
	plt.ylabel('Phase Difference[Deg]')
	plt.xlabel('Frequency [Hz]')
	plt.title('Phase Difference from reference')
	
	plt.suptitle('Difference in various UIM TFs from observed data (used as the reference) in the range: '+str(vec[0])+' Hz to '+str(vec[1])+' Hz')
	plt.show()


