#!/usr/bin/python

# this is a script meant to download data from the time of a braodband pcal injection and apply the corrections to the pcal channel and CALCS to check the calibration.

import os
from scipy import signal,io
import numpy as np
from collections import namedtuple
from math import copysign
import sys
import inspect
from matplotlib import pyplot as plt
SVNROOT = '/ligo/svncommon/CalSVN/aligocalibration/trunk/'
sys.path.insert(0, SVNROOT + 'Common/pyDARM')
from src.computeDARM import computeDARM
from src.filtermath import serielZPK,parallelZPK,freqrespZPK,dfreqrespZPK
from src.iopdownsamplingfilters import iopdownsamplingfilters
from src.daqdownsamplingfilters import daqdownsamplingfilters
from src.readFilter import readFilterSys
from src.readHardcodedTDCFs import readHardcodedActuationTDCFs

showFigs = True
printFigs = False
resultsDir = '/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/'#SVNROOT + 'Runs/' + run + '/' + IFO + '/Results/CALCS_FE/'
dateStr = '2020-01-14'
IFO = 'H1'
run = 'O3'

def tfdiff(reference_complex_tf,new_complex_tf):
	gain_reference_complex_tf = [None] *len(flist)
	pha_reference_complex_tf = [None] *len(flist)
	ratios = [None] *len(flist)
	magdiff = [None] *len(flist)
	phadiff = [None] *len(flist)
	for row in range(len(flist)):
		gain_reference_complex_tf[row] = np.abs(reference_complex_tf[row])
		pha_reference_complex_tf[row] = np.angle(reference_complex_tf[row], deg=True)
		ratios[row] = new_complex_tf[row] / reference_complex_tf[row]
		magdiff[row] = np.abs(ratios[row])
		phadiff[row] = np.angle(ratios[row], deg=True) #this wraps but is ok for tiny angles
	#Convert to Deg, and Unwrap phases... or as presently: don't
	#pha_reference_complex_tf = (180.0 / np.pi) * pha_reference_complex_tf#np.unwrap(pha_reference_complex_tf)* 180.0 / np.pi
	return np.array(gain_reference_complex_tf), np.array(pha_reference_complex_tf), np.array(magdiff), np.array(phadiff)


#flist is identical for the following 3 at file creation, and this is deliberate
flist, L1_foton_new_reallist, L1_foton_new_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/ETMX_L1_new.txt').T
flist, L2_foton_new_reallist, L2_foton_new_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/ETMX_L2_new.txt').T
flist, L3_foton_new_reallist, L3_foton_new_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/ETMX_L3_new.txt').T
flist, L1_foton_old_reallist, L1_foton_old_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/ETMX_L1_old.txt').T
flist, L2_foton_old_reallist, L2_foton_old_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/ETMX_L2_old.txt').T
flist, L3_foton_old_reallist, L3_foton_old_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/2/ETMX_L3_old.txt').T

L1_foton_new = [None] *len(flist)
L2_foton_new = [None] *len(flist)
L3_foton_new = [None] *len(flist)
L1_foton_old = [None] *len(flist)
L2_foton_old = [None] *len(flist)
L3_foton_old = [None] *len(flist)
for row in range(len(flist)):
	L1_foton_new[row] = complex(L1_foton_new_reallist[row],L1_foton_new_imaglist[row])
	L2_foton_new[row] = complex(L2_foton_new_reallist[row],L2_foton_new_imaglist[row])
	L3_foton_new[row] = complex(L3_foton_new_reallist[row],L3_foton_new_imaglist[row])
	L1_foton_old[row] = complex(L1_foton_old_reallist[row],L1_foton_old_imaglist[row])
	L2_foton_old[row] = complex(L2_foton_old_reallist[row],L2_foton_old_imaglist[row])
	L3_foton_old[row] = complex(L3_foton_old_reallist[row],L3_foton_old_imaglist[row])

gain_L1, pha_L1, magdiff_L1, phadiff_L1 = tfdiff( L1_foton_new, L1_foton_old)
gain_L2, pha_L2, magdiff_L2, phadiff_L2 = tfdiff( L2_foton_new, L2_foton_old)
gain_L3, pha_L3, magdiff_L3, phadiff_L3 = tfdiff( L3_foton_new, L3_foton_old)

stages = ['L1', 'L2', 'L3']
gains = [gain_L1, gain_L2, gain_L3]
phases = [pha_L1, pha_L2, pha_L3]
magdiff = [magdiff_L1, magdiff_L2, magdiff_L3]
phadiff = [phadiff_L1, phadiff_L2, phadiff_L3]
fotonold  = [L1_foton_old, L2_foton_old, L3_foton_old]
nDataSets = len(stages)

for n in range(len(stages)):
	cm = plt.get_cmap('gist_rainbow')
	plt.figure(figsize=(15,10)).subplots_adjust(hspace=0.3, wspace=0.2)
	plt.subplot(221)
	plt.xscale('log')
	plt.yscale('log')
	plt.xlim(0.1,2000)
	plt.scatter(flist,gains[n], color='red')#cm(n/nDataSets))
	plt.scatter(flist,np.abs(fotonold[n]), color='blue')#cm(n/nDataSets))
	plt.ylabel('Magnitude[abs]')
	plt.xlabel('Frequency [Hz]')
	plt.grid(True, 'both')
	plt.title('Magnitude of '+stages[n]+' response')
	plt.legend()

	plt.subplot(223)
	plt.xscale('log')
	plt.xlim(0.1,2000)
	plt.scatter(flist,(phases[n]), color='red')#cm(n/nDataSets))
	plt.scatter(flist,np.angle(fotonold[n], deg=True), color='blue')#cm(n/nDataSets))
	plt.ylabel('Phase[rad]')
	plt.xlabel('Frequency [Hz]')
	plt.grid(True, 'both')
	plt.title('Phase of '+stages[n]+' response')

	plt.subplot(222)
	plt.xscale('log')
	#plt.yscale('log')
	plt.xlim(0.1,2000)
	plt.scatter(flist,(magdiff[n]), color='red')#cm(n/nDataSets))
	plt.grid(True, 'both')
	if n==0:
		plt.ylim(1-0.3,1+0.3)
		plt.yticks(np.arange(1-0.3, 1+0.3, 0.05))
	else:
		plt.ylim(1-0.15,1+0.15)
		plt.yticks(np.arange(1-0.15, 1+0.15, 0.01))
	plt.ylabel('Magnitude Ratio')
	plt.xlabel('Frequency [Hz]')
	plt.title('Magnitude Ratio from reference')

	plt.subplot(224)
	plt.xscale('log')
	plt.scatter(flist,(phadiff[n]), color='red')#cm(n/nDataSets))
	plt.grid(True, 'both')
	plt.ylim(-10,10)
	plt.yticks(np.arange(-10, 10, 2))
	plt.xlim(0.1,2000)
	plt.ylabel('Phase Difference[Deg]')
	plt.xlabel('Frequency [Hz]')
	plt.title('Phase Difference from reference')
	
	plt.suptitle('Difference in various '+stages[n]+' TFs from observed data (used as the reference) in the range: 0.01 to 2000 Hz')
	plt.show()


