#!/usr/bin/python

# this is a script meant to download data from the time of a braodband pcal injection and apply the corrections to the pcal channel and CALCS to check the calibration.

import os
from scipy import signal,io
import numpy as np
from collections import namedtuple
from math import copysign
import sys
import inspect
from matplotlib import pyplot as plt

showFigs = True
printFigs = True

IFO = 'H1'
run = 'O3'

#This gets the root path of the SVN repo
#path = os.path.abspath(__file__)
#dirs = path.split('Runs')
SVNROOT = '/ligo/svncommon/CalSVN/aligocalibration/trunk/'
sys.path.insert(0, SVNROOT + 'Common/pyDARM')
sys.path.insert(1, SVNROOT + 'Runs/' + run + '/' + IFO + '/params')

from src.computeDARM import computeDARM
from src.filtermath import serielZPK,parallelZPK,freqrespZPK,dfreqrespZPK
from src.iopdownsamplingfilters import iopdownsamplingfilters
from src.daqdownsamplingfilters import daqdownsamplingfilters
from src.readFilter import readFilterSys
from src.readHardcodedTDCFs import readHardcodedActuationTDCFs

from modelparams_H1_20190909 import modelPars as modelParsOld
from modelparams_H1_20200103 import modelPars as modelParsNew 
pars_old = [modelParsOld()]
pars_new = [modelParsNew()]

#flist is identical for the following 3 at file creation, and this is deliberate
flist, L1_foton_new_reallist, L1_foton_new_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/ETMX_L1_new.txt').T
flist, L2_foton_new_reallist, L2_foton_new_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/ETMX_L2_new.txt').T
flist, L3_foton_new_reallist, L3_foton_new_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/ETMX_L3_new.txt').T
flist, L1_foton_old_reallist, L1_foton_old_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/ETMX_L1_old.txt').T
flist, L2_foton_old_reallist, L2_foton_old_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/ETMX_L2_old.txt').T
flist, L3_foton_old_reallist, L3_foton_old_imaglist = np.genfromtxt('/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/ETMX_L3_old.txt').T

L1_foton_new = [None] *len(flist)
L2_foton_new = [None] *len(flist)
L3_foton_new = [None] *len(flist)
L1_foton_old = [None] *len(flist)
L2_foton_old = [None] *len(flist)
L3_foton_old = [None] *len(flist)
for row in range(len(flist)):
	L1_foton_new[row] = complex(L1_foton_new_reallist[row],L1_foton_new_imaglist[row])
	L2_foton_new[row] = complex(L2_foton_new_reallist[row],L2_foton_new_imaglist[row])
	L3_foton_new[row] = complex(L3_foton_new_reallist[row],L3_foton_new_imaglist[row])
	L1_foton_old[row] = complex(L1_foton_old_reallist[row],L1_foton_old_imaglist[row])
	L2_foton_old[row] = complex(L2_foton_old_reallist[row],L2_foton_old_imaglist[row])
	L3_foton_old[row] = complex(L3_foton_old_reallist[row],L3_foton_old_imaglist[row])

resultsDir = '/ligo/home/vladimir.bossilkov/Work_Done/20200111_calibration_checks/5/plots/'#SVNROOT + 'Runs/' + run + '/' + IFO + '/Results/CALCS_FE/'
dateStr = '2020-01-14'

#The following processing code is copied from pyDARM actuation, with the response functions to the UIM, PUM, and TST replaced by the response from the foton filters.
def computeSUSfilter(filt):
    sysd = signal.ZerosPolesGain([],[],1,dt=1.0/2**14)
    pfilt = []
    for filter in filt:
        [tmp,pfilt] = readFilterSys(filter.filename, filter.name, filter.modules, data=pfilt)
        tmp.gain *= filter.gain
        sysd.zeros = np.append(sysd.zeros, tmp.zeros)
        sysd.poles = np.append(sysd.poles, tmp.poles)
        sysd.gain *= tmp.gain
    return sysd

def act_resp_eval(pars, freq, L1_freqresp, L2_freqresp, L3_freqresp):
##
    # Digital
    ##
    #Compute delays (see G1601472)
    # 1 16k cycle from OMC to SUS transfer
    # 1 16k cycle from SUS to IOP
    OMC2SUSdelay = signal.ZerosPolesGain([],np.zeros(pars.umodelDelayCycles),1,dt=1.0/2**14)
    SUS2IOPdelay = signal.ZerosPolesGain([],np.zeros(pars.umodelDelayCycles),1,dt=1.0/2**14)
    totalUserDelay = serielZPK(OMC2SUSdelay,SUS2IOPdelay)
    #[w,totalUserDelay_freqresp] = signal.dfreqresp(totalUserDelay, 2.0*np.pi*freq/2**14) #performs badly in old scipy versions
    [w,totalUserDelay_freqresp] = dfreqrespZPK(totalUserDelay, 2.0*np.pi*freq/2**14)  #replaced with this

    #Digital filters
    # output is a zpk object
    umodelTST = computeSUSfilter(pars.TSTfilters)
    umodelPUM = computeSUSfilter(pars.PUMfilters)
    umodelUIM = computeSUSfilter(pars.UIMfilters)
    #performs badly in old scipy versions
    #[w,umodelTST_freqresp] = signal.dfreqresp(umodelTST, 2.0*np.pi*freq/2**14)
    #[w,umodelPUM_freqresp] = signal.dfreqresp(umodelPUM, 2.0*np.pi*freq/2**14)
    #[w,umodelUIM_freqresp] = signal.dfreqresp(umodelUIM, 2.0*np.pi*freq/2**14)
    #replaced with this
    [w,umodelTST_freqresp] = dfreqrespZPK(umodelTST, 2.0*np.pi*freq/2**14)
    [w,umodelPUM_freqresp] = dfreqrespZPK(umodelPUM, 2.0*np.pi*freq/2**14)
    [w,umodelUIM_freqresp] = dfreqrespZPK(umodelUIM, 2.0*np.pi*freq/2**14)

    #AI filters
    AIdigital = iopdownsamplingfilters(pars.AIrateString, pars.AImethod, pars.RCGversion)
    AIdigital = AIdigital.to_zpk()

    #DAC delay (see G1601472)
    # 3 IOP compute
    # 1 ZOH and DAC clocked half cycle offset from ADC
    # FIXME (1) move up to where the rest of the delays are computed. JSK 2019-04-04
    # This is supposed to be in roughly the order they appear in the subway map diagram G1501518 EG 2019-07-10
    actIOPdelay = signal.ZerosPolesGain([],np.zeros(pars.iopDelayCycles),1,dt=1.0/2**16)

    #Total digital AI and IOP delay
    # FIXME (2) add OMC2SUSdelay and SUS2IOPdelays here, instead of multiplying it in separately later,
    # This is supposed to be in roughly the order they appear in the subway map diagram G1501518 EG 2019-07-10
    # FIXME and rename this something like "totalDigitalDelays" to keep for use with high frequency corrections. JSK 2019-04-04
    # Suggested fix to combine things at end after definition EG 2019-07-10
    totalActuationDigital = serielZPK(AIdigital, actIOPdelay)
    #[w,totalActuationDigital_freqresp] = signal.dfreqresp(totalActuationDigital, 2.0*np.pi*freq/2**16) #performs badly in old scipy versions
    [w,totalActuationDigital_freqresp] = dfreqrespZPK(totalActuationDigital, 2.0*np.pi*freq/2**16)  #replaced with this

    ## SUS digital filtering * OMC-to-SUS delay * SUS-to-IOP delay * DAC delay * digital AI ##
    ## FIXME (3) then multiply in the digital distribution filters here. JSK 2019-04-04
    # Suggested fix to combine things at end after definition EG 2019-07-10
    umodelTST_freqresp *= totalUserDelay_freqresp * totalActuationDigital_freqresp
    umodelPUM_freqresp *= totalUserDelay_freqresp * totalActuationDigital_freqresp
    umodelUIM_freqresp *= totalUserDelay_freqresp * totalActuationDigital_freqresp

    ##
    # Analog
    ##
    #AI filter
    #read in mat file with zeros poles gain and delay of AA filter
    mat = io.loadmat(pars.aifilter)
    AIanalog = signal.ZerosPolesGain(mat['AAzeros'][:,0],mat['AApoles'][:,0],mat['AAgain'][0,0])
    AIanalog_delay = np.exp( -2.0*np.pi*mat['AAdelay'][0,0]*1j*freq)
    [w, AIanalog_freqresp] = signal.freqresp(AIanalog, 2.0*np.pi*freq)
    AIanalog_freqresp *= AIanalog_delay

    #UIM driver response
    uimDriverResponse = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.UIMdriver.zeros), -2.0*np.pi*np.asarray(pars.UIMdriver.poles), np.prod(2.0*np.pi*np.asarray(pars.UIMdriver.poles))/np.prod(2.0*np.pi*np.asarray(pars.UIMdriver.zeros)))
    #PUM driver response
    if np.ndim(pars.PUMdriver.zeros) == 1:
        # use the predefined average of channels
        pumDriverResponse = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.PUMdriver.zeros), -2.0*np.pi*np.asarray(pars.PUMdriver.poles), np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.poles))/np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.zeros)))
    else: 
        # Take the average of 4 channels as provided
        pumDriver0 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.PUMdriver.zeros[0]), -2.0*np.pi*np.asarray(pars.PUMdriver.poles[0]), np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.poles[0]))/np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.zeros[0])))
        pumDriver1 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.PUMdriver.zeros[1]), -2.0*np.pi*np.asarray(pars.PUMdriver.poles[1]), np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.poles[1]))/np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.zeros[1])))
        pumDriver2 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.PUMdriver.zeros[2]), -2.0*np.pi*np.asarray(pars.PUMdriver.poles[2]), np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.poles[2]))/np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.zeros[2])))
        pumDriver3 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.PUMdriver.zeros[3]), -2.0*np.pi*np.asarray(pars.PUMdriver.poles[3]), np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.poles[3]))/np.prod(2.0*np.pi*np.asarray(pars.PUMdriver.zeros[3])))
        pumDriverResponse = parallelZPK(parallelZPK(parallelZPK(pumDriver0, pumDriver1), pumDriver2), pumDriver3)
        pumDriverResponse.gain *= 0.25
    #TST driver response
    tstDriver0 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.TSTdriver.zeros[0]), -2.0*np.pi*np.asarray(pars.TSTdriver.poles[0]), np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.poles[0]))/np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.zeros[0])))
    tstDriver1 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.TSTdriver.zeros[1]), -2.0*np.pi*np.asarray(pars.TSTdriver.poles[1]), np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.poles[1]))/np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.zeros[1])))
    tstDriver2 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.TSTdriver.zeros[2]), -2.0*np.pi*np.asarray(pars.TSTdriver.poles[2]), np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.poles[2]))/np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.zeros[2])))
    tstDriver3 = signal.ZerosPolesGain(-2.0*np.pi*np.asarray(pars.TSTdriver.zeros[3]), -2.0*np.pi*np.asarray(pars.TSTdriver.poles[3]), np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.poles[3]))/np.prod(2.0*np.pi*np.asarray(pars.TSTdriver.zeros[3])))
    tstDriverResponse = parallelZPK(parallelZPK(parallelZPK(tstDriver0, tstDriver1), tstDriver2), tstDriver3)
    tstDriverResponse.gain *= 0.25

    #Actuator gains
    # FIXME (4) response of each driver without absolute gain is needed further down in the the code,
    # FIXME and yet the N/ct is *= in here, over writing it. Should just save the driver 
    # FIXME response without the N/ct multiplied in. JSK 2019-04-04
    # Can't quite do this as we need the purely frequency dependent part to be removed removing only the absolute gain EG 2019-07-10

    #UIM DC gain
    uimDCgain = pars.UIMactuator.number * \
                pars.UIMactuator.eul2osemcoeff * \
                pars.DACgain * \
                pars.UIMdriver.dcTrans_ApV * \
                pars.UIMactuator.NpA
    uimDriverResponse.gain *= uimDCgain
    # In the CAL-CS model, actsign is +1
    #                      armsign ==> X=+1, Y=-1

    #PUM DC gain
    pumDCgain = pars.PUMactuator.number * \
                pars.PUMactuator.eul2osemcoeff * \
                pars.DACgain * \
                pars.PUMdriver.dcTrans_ApV * \
                pars.PUMactuator.NpA * \
                pars.PUMdriver.coiloutf_sign_flip
    pumDriverResponse.gain *= pumDCgain
    # In the CAL-CS model, actsign is +1
    #                      coilflip ==> LHO=+1, LLO=-1
    #                      armsign ==> X=+1, Y=-1

    #TST DC gain
    #For the TST, we need to know whether the linearization is 'ON' or 'OFF'
    #Here, if the linearization anything other than 'OFF', we exit with a failure message
    # LHO alog 21739
    # in this case applied force is
    # F  = - k (Vc - Vb)^2  #the total force is always towards the reaction mass (k is a positive number), in local coord it is negative
    #    = - k (Vc^2 - 2*Vc*Vb + Vb^2)  % where we apply only linear term
    #
    # F' = (- k) * (-2*Vc*Vb) where Vb = bias voltage
    #
    # JCD (Jenne) 1 Apr 2019: The ESD force is of the same sign as the OSEMs (for a positive bias voltage), so there should not be a minus 
    # sign for "attractive".  
    # Also, there should be no -1 for the linearization.  These 2 minus sign errors cancel, so not giving wrong answer, but 
    # it still should be fixed. We have removed both minus signs in the expression for tstDCgain below 
    
    if pars.linearization!='OFF': sys.exit('TST linearization has been set to a value other than OFF, which is not a currently accepted input')
    if pars.DARMfeedback[3]=='X':
        alpha_NpV2 = pars.TSTactuator.alpha_NpV2[0]
    else:
        alpha_NpV2 = pars.TSTactuator.alpha_NpV2[1]
    tstDCgain = pars.TSTactuator.number * \
                pars.TSTactuator.eul2osemcoeff * \
                pars.DACgain * \
                pars.TSTdriver.dcGain_VpV_lv * \
                alpha_NpV2 * 2 * \
                pars.TSTdriver.dcGain_VpV_hv * pars.AesdBiasVoltage # The actual bias voltge is the AesdBiasVoltage (usually +/-9.5V times HV driver gain) #  dcGain_VpV_lv is the gain in the low voltage path that is applied to the signal (Vc)
    tstDriverResponse.gain *= tstDCgain #tstDriverResponse is set to be 1 above

    #Save driver responses without the absolute gain (absolute gain = Newtons / ct)
    # FIXME (4) response of each driver without absolute gain already has been computed above,
    # FIXME but then mulitpied in using the *=, over writing it. Should just save the driver 
    # FIXME response (only once!) without the N/ct multiplied in. JSK 2019-04-04
    # Can't quite do this as we need the purely frequency dependent part to be removed removing only the absolute gain
    # because we want to obtain the N/ct gain. Should move these computations further down? EG 2019-07-10
    uimDriverResponse_noNpct = uimDriverResponse.to_zpk()
    uimDriverResponse_noNpct.gain /= abs(uimDCgain)
    pumDriverResponse_noNpct = pumDriverResponse.to_zpk()
    pumDriverResponse_noNpct.gain /= abs(pumDCgain)
    tstDriverResponse_noNpct = tstDriverResponse.to_zpk()
    tstDriverResponse_noNpct.gain /= abs(tstDCgain)  #instead of doing all this, we could have just used tstDriverResponse which has a DC gain of 1 (see FIXME above)

    #TOP response
    # FIXME (5) the top mass (if it ever gets used) should be treated in the same way as the other drivers
    # FIXME i.e. its parameters should not be hard-coded here, 
    # FIXME but we should probably just remove it since it's excess clutter and not used anywhere. JSK 2019-04-04
    topResponse = signal.ZerosPolesGain([],[],0)
    #[w,topResponse_freqresp] = signal.freqresp(topResponse, 2.0*np.pi*freq)  #old versions of scipy complain about this
    topResponse_freqresp = freqrespZPK(topResponse, 2.0*np.pi*freq)       #use this instead to silence complaints
    #UIM response
    [w,uimResponse_freqresp] = signal.freqresp(uimDriverResponse, 2.0*np.pi*freq)
    #PUM response
    [w,pumResponse_freqresp] = signal.freqresp(pumDriverResponse, 2.0*np.pi*freq)
    #TST response
    [w,tstResponse_freqresp] = signal.freqresp(tstDriverResponse, 2.0*np.pi*freq)

    ###################################################################################################################################################################################################################################################Here we insert our foton TFs
    UIM_F2L_func = L1_freqresp
    PUM_F2L_func = L2_freqresp
    TST_F2L_func = L3_freqresp

    tstResponse_freqresp *= TST_F2L_func
    pumResponse_freqresp *= PUM_F2L_func
    uimResponse_freqresp *= UIM_F2L_func

    # Unknown actuation delay
    # FIXME (6) Put this up near where the rest of the delays ae calculated. JSK 2019-04-04
    # FIXME (7) Should this be a digital delay or an analog delay? (currently an analog delay). JSK 2019-04-04
    unknownActuationDelay = np.exp( -2.0*np.pi*1j * pars.unknownActuationDelay * freq)
    unknownPUMDelay = np.exp( -2.0*np.pi*1j * pars.unknownPUMDelay * freq)
    #
    # Combine the entire actuator
    # FIXME (8) VERY difficult to figure out what's in this.
    # FIXME Rearrage above variables and below construction to flow from DARM_CTRL to displacement
    # FIXME of the test mass. JSK 2019-04-04
    # As of 2019-04-04, these things are:
    # {istage}Response_freqresp = [(overall gain)^{i} [N/ct] * SUSRESP^{i} [m/N] * (high-freq driver poles)^{i} * (actuation sign convention flaw)^{i}]
    # umodel{ISTAGE}_freqresp   = [OMC2SUS delay * SUS2IOP delay * IOP2Analog delay * AI(D) * (LOCK Filters)^{i} * (DRIVEALIGN Filters)^{i}]
    # AIanalog_freqresp         = [AI(A)]
    # unknownActuationDelay     = [unkown delay]
    TOP = topResponse_freqresp * AIanalog_freqresp * unknownActuationDelay
    TST = tstResponse_freqresp * umodelTST_freqresp * AIanalog_freqresp * unknownActuationDelay
    PUM = pumResponse_freqresp * umodelPUM_freqresp * AIanalog_freqresp * unknownActuationDelay * unknownPUMDelay
    UIM = uimResponse_freqresp * umodelUIM_freqresp * AIanalog_freqresp * unknownActuationDelay
    
    ##For the EPICS records
    # FIXME (9) Should not need to reconstruct something special for EPICS records if construction of A is done 
    # FIXME from DARM_CTRL to displacement. See above fix about construction of overall actuator. JSK 2019-04-04
    # Calibration lines are injected at the input to the DRIVEALIGN bank, so we need the path from that point to meters
    # FIXME (14) Why doesn't commonTF have the OMC-SUS delay, totalUserDelay_freqresp? Seems like it should! JSK 2019-04-04
    # FIXME Answer -- in the definition of EPICs records, here, the OMC2SUS (1 clock cycle, 61 us) delay required in the 
    # FIXME actual DARM loop from the omc model down to the end station SUS model is cancelled by the OMC2CALCS 
    # FIXME advance needed to account the delay accrued from jump from in-loop DARM_CTRL to the CALCS demodulators 
    # FIXME and/or because the GDS pipeline uses CAL-CS_DARM_CTRL. We thus sneakily just leave both terms out.
    # FIXME We should fix this sneakiness.
    #                   SUS-IOP Delay                                   AI(D)*IOP-Analog Delay            AI(A)               Unknown Delay
    commonTF = dfreqrespZPK(SUS2IOPdelay, 2.0*np.pi*freq/2**14)[1] * totalActuationDigital_freqresp * AIanalog_freqresp * unknownActuationDelay
    #Adding stage specific TFs (this allows us to add individual TST high frequency corrections for example) JCB
    tstTF = 1.0
    pumTF = unknownPUMDelay
    uimTF = 1.0
    # From ISCINF input to output of LOCK
    umodelTST_thruLock_freqresp = dfreqrespZPK(computeSUSfilter(pars.TSTfilters[:2:]), 2.0*np.pi*freq/2**14)[1]
    umodelPUM_thruLock_freqresp = dfreqrespZPK(computeSUSfilter(pars.PUMfilters[:3:]), 2.0*np.pi*freq/2**14)[1]
    umodelUIM_thruLock_freqresp = dfreqrespZPK(computeSUSfilter(pars.UIMfilters[:4:]), 2.0*np.pi*freq/2**14)[1]
    # LOCK input to output
    umodelTST_lock_freqresp = dfreqrespZPK(computeSUSfilter([pars.TSTfilters[1]]), 2.0*np.pi*freq/2**14)[1]
    umodelPUM_lock_freqresp = dfreqrespZPK(computeSUSfilter([pars.PUMfilters[2]]), 2.0*np.pi*freq/2**14)[1]
    umodelUIM_lock_freqresp = dfreqrespZPK(computeSUSfilter([pars.UIMfilters[3]]), 2.0*np.pi*freq/2**14)[1]
    # DRIVEALIGN and anything in OUTPUT filter banks
    umodelTST_afterLock = computeSUSfilter(pars.TSTfilters[2::])
    umodelPUM_afterLock = computeSUSfilter(pars.PUMfilters[3::])
    umodelUIM_afterLock = computeSUSfilter(pars.UIMfilters[4::])
    umodelTST_afterLock_freqresp = dfreqrespZPK(umodelTST_afterLock, 2.0*np.pi*freq/2**14)[1]
    umodelPUM_afterLock_freqresp = dfreqrespZPK(umodelPUM_afterLock, 2.0*np.pi*freq/2**14)[1]
    umodelUIM_afterLock_freqresp = dfreqrespZPK(umodelUIM_afterLock, 2.0*np.pi*freq/2**14)[1]

    # EPICS records
    # Adding in new stage specific "analog" TFs JCB
    #              see above  DRIVEALIGN (and anything else)    driver response * pendulum response
    EPICS_A_TST_0 = tstTF * commonTF * umodelTST_afterLock_freqresp * tstResponse_freqresp    #This is the first part of EP1 in T1700106
    EPICS_A_PUM_0 = pumTF * commonTF * umodelPUM_afterLock_freqresp * pumResponse_freqresp    #This is the first part of EP2 in T1700106
    EPICS_A_UIM_0 = uimTF * commonTF * umodelUIM_afterLock_freqresp * uimResponse_freqresp    #This is the first part of EP3 in T1700106
    #The sign of the EPICS record 

    # FIXME (10) the "armSign" starts to appear here, and it's applied separately for the EPICS
    # FIXME records and for the full actuator transfer function. The application should be
    # FIXME done once and applied in a way that's common to both. See above FIXME (8) above about
    # FIXME rearranging the construction of A. JSK 2019-04-04
    #A positive gain on DRIVEALIGN results in shorter arm cavity. See T1800456
    #For X-arm, this means DARM is negative, while Y-arm means DARM is positive.
    #Thus, for X-arm SUS actuators, we need to flip the sign of the EPICS record by multiplying by -1
    if pars.DARMfeedback[1]=='X': EPICS_A_UIM_0 *= -1.0
    if pars.DARMfeedback[2]=='X': EPICS_A_PUM_0 *= -1.0
    if pars.DARMfeedback[3]=='X': EPICS_A_TST_0 *= -1.0

    #Actuation from SUS TEST excitation point to displacement
    #See G1601472 for timing and G1501518 for schematic diagram
    #Adding in new stage specific "analog" TFs JCB
    #Note that we include the unknown actuation delay from the model, so if there is any delay in the MCMC it should go into this parameter
    #                       see above    normalized driver frequency response                 force to length pendulum response
    TSTTESTexc2DARM_noNpct = tstTF * commonTF * signal.freqresp(tstDriverResponse_noNpct, 2.0*np.pi*freq)[1] * TST_F2L_func
    PUMTESTexc2DARM_noNpct = pumTF * commonTF * signal.freqresp(pumDriverResponse_noNpct, 2.0*np.pi*freq)[1] * PUM_F2L_func
    UIMTESTexc2DARM_noNpct = uimTF * commonTF * signal.freqresp(uimDriverResponse_noNpct, 2.0*np.pi*freq)[1] * UIM_F2L_func
    #Actuation from SUS CAL excitation point (at input to DRIVEALIGN bank)
    #                       see above              DRIVEALIGN BANK (and anything else)                       
    TSTCALexc2DARM_noNpct = TSTTESTexc2DARM_noNpct * umodelTST_afterLock_freqresp
    PUMCALexc2DARM_noNpct = PUMTESTexc2DARM_noNpct * umodelPUM_afterLock_freqresp
    UIMCALexc2DARM_noNpct = UIMTESTexc2DARM_noNpct * umodelUIM_afterLock_freqresp
    #Like EPICS, need to multiply by the driven arm's impact on DARM
    if pars.DARMfeedback[1]=='X':
        UIMTESTexc2DARM_noNpct *= -1.0
        UIMCALexc2DARM_noNpct *= -1.0
    if pars.DARMfeedback[2]=='X':
        PUMTESTexc2DARM_noNpct *= -1.0
        PUMCALexc2DARM_noNpct *= -1.0
    if pars.DARMfeedback[3]=='X':
        TSTTESTexc2DARM_noNpct *= -1.0
        TSTCALexc2DARM_noNpct *= -1.0

    #CAL-CS_DARM_DELTAL_CTRL to GDS inverse sensing filter output
    #These are the remaining things that are not compensated by front end filters in CAL-CS
    #See G1501518 and G1601472
    #In the past, GDS code pulled the 'NoDAQDownSample' selection itself and created a DAQ downsampling filter as well as the 1 clock cycle delay
    #I humbly offer the calcs2gdsResidActFiltOut variable that includes the DAQDownSamplingFilter and this 1 clock cycle delay
    # FIXME (11) Lots of repitition of computation here, which makes it all prone to errors. Let's
    # FIXME again construct these with more commonly used variables, and also break out the 
    # FIXME convoluted combinations of variables aligned with FIXME (8) above, and/or just use
    # FIXME the complete actuator function and then apply the corrections for GDS. JSK 2019-04-04
    # FIXME (15) Why don't calcs2gdsResidActFiltOut include high frequency driver poles (specifically for TST), {iStage}DriverResponse_noNpct? Seems like they should! JSK 2019-04-04
    #                                                AI(D)*IOP-Analog Delay           AI(A)         SUS-OMC & SUS-IOP delay      Unknown Delay
    calcs2gdsResidActFiltOut_NoDAQDownSample = totalActuationDigital_freqresp * AIanalog_freqresp * totalUserDelay_freqresp * unknownActuationDelay
    # Now advance from CAL-CS_DARM_CTRL back to LSC-DARM_CTRL, by dividing "OMC2SUSdelay" which is actually a OMC2CALCSdelay
    calcs2gdsResidActFiltOut_NoDAQDownSample_removeCALCSdelay = totalActuationDigital_freqresp * \
                                                                AIanalog_freqresp * \
                                                                totalUserDelay_freqresp * \
                                                                unknownActuationDelay / \
                                                                dfreqrespZPK(OMC2SUSdelay, 2.0*np.pi*freq/2**14)[1]
    # Now do both advancing back to LSC-DARM_CTRL and dividing out the 16k to 4k DAQ downsampling filter
    calcs2gdsResidActFiltOut = calcs2gdsResidActFiltOut_NoDAQDownSample / dfreqrespZPK(daqdownsamplingfilters(2**14,2**12,'biquad','v3').to_zpk(), 2.0*np.pi*freq/2**14)[1] / dfreqrespZPK(OMC2SUSdelay, 2.0*np.pi*freq/2**14)[1]
    
    # DARM_CTRL to Mass    
    # Now include the LSC output matrix elements 
    if pars.DARMfeedback[0]=='X': darm_ctrl_to_top = pars.DARMoutputMatrix[0]
    else: darm_ctrl_to_top = pars.DARMoutputMatrix[1]
    if pars.DARMfeedback[1]=='X': darm_ctrl_to_uim = pars.DARMoutputMatrix[0]
    else: darm_ctrl_to_uim = pars.DARMoutputMatrix[1]
    if pars.DARMfeedback[2]=='X': darm_ctrl_to_pum = pars.DARMoutputMatrix[0]
    else: darm_ctrl_to_pum = pars.DARMoutputMatrix[1]
    if pars.DARMfeedback[3]=='X': darm_ctrl_to_tst = pars.DARMoutputMatrix[0]
    else: darm_ctrl_to_tst = pars.DARMoutputMatrix[1]

    TOP *= darm_ctrl_to_top
    UIM *= darm_ctrl_to_uim
    PUM *= darm_ctrl_to_pum
    TST *= darm_ctrl_to_tst

    # Now take care of the "arm sign" this is to transform from local suspension coordinates, 
    # where +L is defined as moving the optic perpendicular to the highly reflective surface, 
    # to DARM coordinates, DARM = Lx-Ly.  
    # Since +L on ETMX makes the X arm shorter, it makes DARM smaller, so the arm sign for ETMX is -1.
    # Since +L on ETMY makes the Y arm shorter, it makes DARM larger, so the arm sign for ETMY is +1 
    # For ETMY, the armSign implicitly "included" here by *not* multiplying by -1.
    if pars.DARMfeedback[0]=='X': TOP *= -1
    if pars.DARMfeedback[1]=='X': UIM *= -1
    if pars.DARMfeedback[2]=='X': PUM *= -1
    if pars.DARMfeedback[3]=='X': TST *= -1



    # Everything *above* here is modelling the physical actuator that we have.  However, the Calibration 
    # group's convention is that A in their loop drawings is minus the physical actuator (which allows the 
    # explicit writing of the minus sign in the loop drawing to indicate negative feedback).  
    # Ideally we'd just put A = -1 * (TOP + UIM + PUM + TST), but the actuation products below 
    # (which are used for other things downstream) are expecting to receive minus of each stage 
    # of the physical actuator.
    # So, we multiply every stage by -1, and then sum them up, and the result will be that A is negative 
    # of the physical actuator.  JCD_26Mar2019
    # See more details in T1800456.
    # FIXME (12) related to FIXME (10), we should be applying the armSign in single place, consistently
    TOP *= -1
    UIM *= -1
    PUM *= -1
    TST *= -1

    #If the above sign convention still confuses you (and don't be afraid, Evan is confused by this regularly), try T1800456
    return TOP, UIM, PUM, TST







TOP, UIM, PUM, TST = act_resp_eval(pars_new[0].Apars ,flist, L1_foton_new ,L2_foton_new ,L3_foton_new)
TOPold, UIMold, PUMold, TSTold = act_resp_eval(pars_old[0].Apars ,flist, L1_foton_old ,L2_foton_old ,L3_foton_old)

Afotnonew = TOP + UIM + PUM + TST
Afotnoold = TOPold + UIMold + PUMold + TSTold

[D_pydarmnew, C_pydarmnew, A_pydarmnew, G_pydarmnew, CLG_new, R_pydarmnew, sensProd_pydarmnew, actProd_pydarmnew] = computeDARM(pars_new[0], flist)

R_fotonnew = (1.0 + C_pydarmnew * D_pydarmnew * Afotnonew)/C_pydarmnew
R_fotonold = (1.0 + C_pydarmnew * D_pydarmnew * Afotnoold)/C_pydarmnew

ratio = [None] *len(flist)
for row in range(len(flist)):
	ratio[row] = R_fotonnew[row]/R_fotonold[row]

#####  Figure #######
SMALL_SIZE = 12
MEDIUM_SIZE = 13
BIGGER_SIZE = 15

plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
plt.rc('axes', titlesize=SMALL_SIZE)     # fontsize of the axes title
plt.rc('axes', labelsize=MEDIUM_SIZE)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=SMALL_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=SMALL_SIZE)    # legend fontsize
plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title

fig = plt.figure(figsize=(15,10))
s1 = fig.add_subplot(211)
s1.plot(flist,np.abs(ratio),linestyle=':',color='r',label='R_new / R_old')
s1.set_xlim(5, 2000)
s1.set_ylim(0.96, 1.04)
s1.set_yticks(np.arange(0.96,1.04,0.01))
s1.set_xscale('log')
s1.legend(ncol=1,fontsize='x-small')
s1.minorticks_on()
s1.grid(ls='-', which='major',lw=1.1)
s1.grid(ls='--', which='minor')
plt.suptitle('Ratio between: new R_foton (generated using foton filters for DELTAL)\n and old R_foton (generated using foton filters for DELTAL)')
plt.tight_layout(rect=[0, 0.03, 1, 0.95])

s2 = fig.add_subplot(212)
s2.plot(flist,(180.0/np.pi)*np.angle(ratio),linestyle=':',color='r',label='R_new / R_old')
s2.set_xlabel('Frequency (Hz)')
s2.set_ylabel('Phase (deg)')
s2.set_xlim(5, 2000)
s2.set_ylim(-3, 3)
s2.set_yticks(np.arange(-3,3,1))
s2.set_xscale('log')
s2.minorticks_on()
s2.grid(ls='-', which='major',lw=1.1)
s2.grid(ls='--', which='minor')

if printFigs:
    thisfig = resultsDir + dateStr + '_' + IFO +  '_Rfoton_new_over_Rfoton_old.pdf'
    plt.savefig(thisfig, bbox_inches='tight', pad_inches=0.2)
if showFigs: plt.show()


